/*******************  start of original comments  ********************/
/*
 * Written by Douglas Thomson (1989/1990)
 *
 * This source code is released into the public domain.
 */
/*********************  end of original comments   ********************/

/*
 * New editor name:  tde, the Thomson-Davis Editor.
 * Author:           Frank Davis
 * Date:             June 5, 1991
 *
 * This modification of Douglas Thomson's code is released into the
 * public domain, Frank Davis.  You may distribute it freely.
 *
 * See "tdestr.h" for a description of these variables. C requires global
 *  variables to be declared "extern" in all modules except one.  This file
 *  is included in ed.c and it should not be included in any other module.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <conio.h>
#ifdef __TURBOC__
   #include <stdlib.h>
#elif __MSC__
   #include <stdlib.h>
   #if defined( toupper )
       #undef toupper
   #endif
#endif

displays g_display;

status_infos g_status;

boyer_moore_type bm;

mode_infos mode = {
   0,           /* initial color scheme */
   FALSE,       /* sync mode? */
   TRUE,        /* sync semaphore must be TRUE - DO NOT CHANGE */
   TRUE,        /* insert mode */
   TRUE,        /* indent mode */
   8,           /* tab size */
   FALSE,       /* use backup files - flag is not used */
   FALSE,       /* enhanced keyboard flag */
   "<=======   End of File   =======>",       /* eof message */
   TRUE,        /* default TRUE = do not write ^Z at end of file */
   CRLF,        /* default write <cr><lf> at eol */
   TRUE,        /* default remove trailing space on edited lines */
   FALSE,       /* default show eol character is off */
   FALSE,       /* default word wrap mode is off */
   0,           /* default left margin - add 1 to margins for display */
   2,           /* default paragraph begin */
   71           /* default right margin */
};

/*
 * do_it is an array of pointers to functions that return void with an argument
 * that is a pointer to a window.  Is that right???
 */

#define NUM_FUNCS        122

void (* (do_it[NUM_FUNCS]))( windows * ) = {
   insert_overwrite,                /*   regular text keys        0  */
   get_help,                        /*   Help                     1  */
   insert_newline,                  /*   Rturn                    2  */
   next_line,                       /*   NextLine                 3  */
   beg_next_line,                   /*   BegNextLine              4  */
   move_down,                       /*   LineDown                 5  */
   move_up,                         /*   LineUp                   6  */
   move_right,                      /*   CharRight                7  */
   move_left,                       /*   CharLeft                 8  */
   scroll_right,                    /*   ScrollRight              9  */
   scroll_left,                     /*   ScrollLeft              10  */
   word_right,                      /*   WordRight               11  */
   word_left,                       /*   WordLeft                12  */
   page_down,                       /*   ScreenDown              13  */
   page_up,                         /*   ScreenUp                14  */
   goto_end_file,                   /*   EndOfFile               15  */
   goto_top_file,                   /*   TopOfFile               16  */
   goto_bottom,                     /*   BotOfScreen             17  */
   goto_top,                        /*   TopOfScreen             18  */
   goto_right,                      /*   EndOfLine               19  */
   home,                            /*   BegOfLine               20  */
   goto_line,                       /*   JumpToLine              21  */
   center_window,                   /*   CenterWindow            22  */
   center_window,                   /*   CenterLine              23  */
   horizontal_screen_right,         /*   HorizontalScreenRight   24  */
   horizontal_screen_left,          /*   HorizontalScreenLeft    25  */
   scroll_down,                     /*   ScrollDnLine            26  */
   scroll_up,                       /*   ScrollUpLine            27  */
   fixed_scroll_up,                 /*   FixedScrollUp           28  */
   fixed_scroll_down,               /*   FixedScrollDn           29  */
   toggle_overwrite,                /*   ToggleOverWrite         30  */
   toggle_indent,                   /*   ToggleIndent            31  */
   toggle_ww,                       /*   ToggleWordWrap          32  */
   toggle_crlf,                     /*   ToggleCRLF              33  */
   toggle_trailing,                 /*   ToggleTrailing          34  */
   toggle_z,                        /*   ToggleZ                 35  */
   toggle_eol,                      /*   ToggleEol               36  */
   toggle_sync,                     /*   ToggleSync              37  */
   set_tabstop,                     /*   SetTabs                 38  */
   set_left_margin,                 /*   SetLeftMargin           39  */
   set_right_margin,                /*   SetRightMargin          40  */
   set_paragraph_margin,            /*   SetParagraphMargin      41  */
   format_paragraph,                /*   FormatParagraph         42  */
   format_paragraph,                /*   FormatText              43  */
   left_justify,                    /*   LeftJustify             44  */
   right_justify,                   /*   RightJustify            45  */
   center_justify,                  /*   CenterJustify           46  */
   tab_key,                         /*   Tab                     47  */
   backtab,                         /*   BackTab                 48  */
   match_pair,                      /*   ParenBalance            49  */
   back_space,                      /*   BackSpace               50  */
   char_del_under,                  /*   DeleteChar              51  */
   char_del_under,                  /*   StreamDeleteChar        52  */
   line_kill,                       /*   DeleteLine              53  */
   eol_kill,                        /*   DelEndOfLine            54  */
   word_delete,                     /*   WordDelete              55  */
   insert_newline,                  /*   AddLine                 56  */
   insert_newline,                  /*   SplitLine               57  */
   join_line,                       /*   JoinLine                58  */
   dup_line,                        /*   DuplicateLine           59  */
   undo_line,                       /*   AbortCommand            60  */
   undo_line,                       /*   UndoLine                61  */
   undo,                            /*   UndoDelete              62  */
   toggle_search_case,              /*   ToggleSearchCase        63  */
   find_string,                     /*   FindForward             64  */
   find_string,                     /*   FindBackward            65  */
   find_string,                     /*   RepeatFindForward1      66  */
   find_string,                     /*   RepeatFindForward2      67  */
   find_string,                     /*   RepeatFindBackward1     68  */
   find_string,                     /*   RepeatFindBackward2     69  */
   replace_string,                  /*   ReplaceForward          70  */
   replace_string,                  /*   ReplaceBackward         71  */
   mark_block,                      /*   MarkBlock               72  */
   mark_block,                      /*   MarkLine                73  */
   mark_block,                      /*   MarkStream              74  */
   unmark_block,                    /*   UnMarkBlock             75  */
   move_copy_delete_overlay_block,  /*   FillBlock               76  */
   move_copy_delete_overlay_block,  /*   CopyBlock               77  */
   move_copy_delete_overlay_block,  /*   KopyBlock               78  */
   move_copy_delete_overlay_block,  /*   MoveBlock               79  */
   move_copy_delete_overlay_block,  /*   OverlayBlock            80  */
   move_copy_delete_overlay_block,  /*   DeleteBlock             81  */
   block_write,                     /*   BlockToFile             82  */
   block_print,                     /*   PrintBlock              83  */
   block_expand_tabs,               /*   BlockExpandTabs         84  */
   block_convert_case,              /*   BlockUpperCase          85  */
   block_convert_case,              /*   BlockLowerCase          86  */
   block_convert_case,              /*   BlockStripHiBit         87  */
   edit_another_file,               /*   EditFile                88  */
   file_file,                       /*   File                    89  */
   save_file,                       /*   Save                    90  */
   save_as_file,                    /*   SaveAs                  91  */
   change_fattr,                    /*   SetFileAttributes       92  */
   edit_next_file,                  /*   EditNextFile            93  */
   redraw_screen,                   /*   RedrawScreen            94  */
   size_window,                     /*   SizeWindow              95  */
   split_screen,                    /*   SplitScreen             96  */
   next_window,                     /*   NextWindow              97  */
   prev_window,                     /*   PreviousWindow          98  */
   zoom_window,                     /*   ZoomWindow              99  */
   next_hidden_window,              /*   NextHiddenWindow       100  */
   set_marker,                      /*   SetMark1               101  */
   set_marker,                      /*   SetMark2               102  */
   set_marker,                      /*   SetMark3               103  */
   set_marker,                      /*   SetMark4               104  */
   set_marker,                      /*   SetMark5               105  */
   set_marker,                      /*   SetMark6               106  */
   set_marker,                      /*   SetMark7               107  */
   set_marker,                      /*   SetMark8               108  */
   set_marker,                      /*   SetMark9               109  */
   set_marker,                      /*   SetMark0               110  */
   goto_marker,                     /*   GotoMark1              111  */
   goto_marker,                     /*   GotoMark2              112  */
   goto_marker,                     /*   GotoMark3              113  */
   goto_marker,                     /*   GotoMark4              114  */
   goto_marker,                     /*   GotoMark5              115  */
   goto_marker,                     /*   GotoMark6              116  */
   goto_marker,                     /*   GotoMark7              117  */
   goto_marker,                     /*   GotoMark8              118  */
   goto_marker,                     /*   GotoMark9              119  */
   goto_marker,                     /*   GotoMark0              120  */
   quit                             /*   Quit                   121  */
};
